import {
    AfterViewInit,
    Component,
    ElementRef,
    OnInit,
    ViewChild
} from '@angular/core';

import { ActivatedRoute } from '@angular/router';

import {
    MatPaginator,
    MatSort,
    MatTableDataSource
} from '@angular/material';

import {
    debounceTime,
    distinctUntilChanged,
    startWith,
    tap,
    delay
} from 'rxjs/operators';

import { merge } from 'rxjs/observable/merge';
import { fromEvent } from 'rxjs/observable/fromEvent';

import { ClipsService } from '../services/clips.service';
import { ClipsDataSource } from '../services/clips.datasource';


@Component({
    selector: 'clips',
    templateUrl: './clips.component.html',
    styleUrls: [ './clips.component.css' ]
})
export class ClipsComponent implements OnInit, AfterViewInit {
    dataSource: ClipsDataSource;

    displayedColumns = [
        'created_at',
        'title',
        'curator_display_name',
        'game',
        'duration'
    ];

    @ViewChild(MatPaginator) paginator: MatPaginator;
    @ViewChild(MatSort) sort: MatSort;
    @ViewChild('input') input: ElementRef;

    constructor(private route: ActivatedRoute,
                private clipsService: ClipsService) { }

    ngOnInit() {
        this.dataSource = new ClipsDataSource(this.clipsService);
        this.dataSource.loadClips('', 'created_at', 'desc', 0, 10);
    }

    ngAfterViewInit() {
        this.sort.sortChange.subscribe(() => this.paginator.pageIndex = 0);

        fromEvent(this.input.nativeElement, 'keyup')
            .pipe(
                debounceTime(150),
                distinctUntilChanged(),
                tap(() => {
                    this.paginator.pageIndex = 0;
                    this.loadClipsPage();
                })
            )
            .subscribe();

        merge(this.sort.sortChange, this.paginator.page)
            .pipe(
                tap(() => this.loadClipsPage())
            )
            .subscribe();
    }

    loadClipsPage() {
        this.dataSource.loadClips(
            this.input.nativeElement.value,
            this.sort.active,
            this.sort.direction,
            this.paginator.pageIndex,
            this.paginator.pageSize);
    }
}
