import {
    CollectionViewer,
    DataSource
} from '@angular/cdk/collections';

import { Observable } from 'rxjs/Observable';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import { of } from 'rxjs/observable/of';

import {
    catchError,
    finalize
} from 'rxjs/operators';

import { ClipsService } from './clips.service';

import { Clip } from '../models/clip';


export class ClipsDataSource implements DataSource<Clip> {
    private clipsSubject = new BehaviorSubject<Clip[]>([]);
    private countSubject = new BehaviorSubject<number>(0);
    private loadingSubject = new BehaviorSubject<boolean>(false);

    public count$ = this.countSubject.asObservable();
    public loading$ = this.loadingSubject.asObservable();

    constructor(private clipsService: ClipsService) { }

    loadClips(filter: string,
              sortColumn: string,
              sortDirection: string,
              pageIndex: number,
              pageSize: number) {

        this.loadingSubject.next(true);

        this.clipsService.findClips(filter, sortColumn, sortDirection,
                                    pageIndex, pageSize)
            .pipe(
                catchError(() => of([])),
                finalize(() => this.loadingSubject.next(false))
            )
            .subscribe((data: any) => {
                this.clipsSubject.next(data.clips);
                this.countSubject.next(data.totalCount);
            });
    }

    connect(collectionViewer: CollectionViewer): Observable<Clip[]> {
        console.log('Connecting data source');
        return this.clipsSubject.asObservable();
    }

    disconnect(collectionViewer: CollectionViewer): void {
        this.clipsSubject.complete();
        this.countSubject.complete();
        this.loadingSubject.complete();
    }
}
