import {
    AfterViewInit,
    Component,
    ElementRef,
    OnInit,
    ViewChild
} from '@angular/core';

import { ActivatedRoute } from '@angular/router';

import {
    MatPaginator,
    MatSort,
    MatTableDataSource
} from '@angular/material';

import { DatePipe } from '@angular/common';

import {
    debounceTime,
    distinctUntilChanged,
    startWith,
    tap,
    delay
} from 'rxjs/operators';

import { merge } from 'rxjs/observable/merge';
import { fromEvent } from 'rxjs/observable/fromEvent';

import { GiftedSubsService } from '../services/giftedsubs.service';
import { GiftedSubsDataSource } from '../services/giftedsubs.datasource';


@Component({
    selector: 'gifted-subs',
    templateUrl: './giftedsubs.component.html',
    styleUrls: [ './giftedsubs.component.css' ]
})
export class GiftedSubsComponent implements OnInit, AfterViewInit {
    dataSource: GiftedSubsDataSource;

    displayedColumns = [
        'time',
        'giver',
        'receiver'
    ];

    @ViewChild(MatPaginator) paginator: MatPaginator;
    @ViewChild(MatSort) sort: MatSort;
    @ViewChild('input') input: ElementRef;

    constructor(private route: ActivatedRoute,
                private giftedSubsService: GiftedSubsService) { }

    ngOnInit() {
        this.dataSource = new GiftedSubsDataSource(this.giftedSubsService);
        this.dataSource.loadGiftedSubs('', 'time', 'desc', 0, 20);
    }

    ngAfterViewInit() {
        this.sort.sortChange.subscribe(() => this.paginator.pageIndex = 0);

        fromEvent(this.input.nativeElement, 'keyup')
            .pipe(
                debounceTime(150),
                distinctUntilChanged(),
                tap(() => {
                    this.paginator.pageIndex = 0;
                    this.loadGiftedSubsPage();
                })
            )
            .subscribe();

        merge(this.sort.sortChange, this.paginator.page)
            .pipe(
                tap(() => this.loadGiftedSubsPage())
            )
            .subscribe();
    }

    loadGiftedSubsPage() {
        this.dataSource.loadGiftedSubs(
            this.input.nativeElement.value,
            this.sort.active,
            this.sort.direction,
            this.paginator.pageIndex,
            this.paginator.pageSize);
    }
}
