import {
    CollectionViewer,
    DataSource
} from '@angular/cdk/collections';

import { Observable } from 'rxjs/Observable';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import { of } from 'rxjs/observable/of';

import {
    catchError,
    finalize
} from 'rxjs/operators';

import { MerchandiseService } from './merchandise.service';

import { Product } from '../models/product';


export class MerchandiseDataSource implements DataSource<Product> {
    private productsSubject = new BehaviorSubject<Product[]>([]);
    private countSubject = new BehaviorSubject<number>(0);
    private loadingSubject = new BehaviorSubject<boolean>(false);

    public count$ = this.countSubject.asObservable();
    public loading$ = this.loadingSubject.asObservable();

    constructor(private merchandiseService: MerchandiseService) { }

    loadProducts(filter: string,
               sortColumn: string,
               sortDirection: string,
               pageIndex: number,
               pageSize: number) {

        this.loadingSubject.next(true);

        this.merchandiseService.findProducts(filter, sortColumn, sortDirection,
                                      pageIndex, pageSize)
            .pipe(
                catchError(() => of([])),
                finalize(() => this.loadingSubject.next(false))
            )
            .subscribe(data => {
                this.productsSubject.next(data.products);
                this.countSubject.next(data.totalCount);
            });
    }

    connect(collectionViewer: CollectionViewer): Observable<Product[]> {
        console.log('Connecting data source');
        return this.productsSubject.asObservable();
    }

    disconnect(collectionViewer: CollectionViewer): void {
        this.productsSubject.complete();
        this.countSubject.complete();
        this.loadingSubject.complete();
    }
}
